import java.util.ArrayList;
import java.util.Collections;

public class SortExample
{
    public static int[] merge(int[] list1, int[] list2)
    {
        int[] listOut = new int[list1.length + list2.length];

        int i1 = 0;  // Current position in list 1
        int i2 = 0;
        int iOut = 0;

        // Continue as long as both lists have some item left
        while(i1 < list1.length && i2 < list2.length)
        {
            if(list1[i1] < list2[i2])
            {
                // Take the item from list 1
                listOut[iOut] = list1[i1];
                ++iOut;
                ++i1;
            }
            else
            {
                // Take the item from list 2
                listOut[iOut] = list2[i2];
                ++iOut;
                ++i2;
            }
        }
        // Handle the rest of list 1
        while(i1 < list1.length)
        {
            listOut[iOut] = list1[i1];
            ++iOut;
            ++i1;
        }

        // Handle the rest of list 2
        while(i2 < list2.length)
        {
            listOut[iOut] = list2[i2];
            ++iOut;
            ++i2;
        }

        return listOut;
    }

    public static int[] mergeSort(int[] list)
    {
        if(list.length <= 1)
        {
            // Base case
            return list;
        }

        // Recursive case
        int[] list1 = new int[list.length / 2];
        int[] list2 = new int[(list.length + 1) / 2]; 

        // Split list into two halves
        for(int i = 0; i < list.length/2; ++i)
        {
            list1[i] = list[i];  // First half
            list2[i] = list[i + list.length / 2];  // Second half
        }

        if(list.length % 2 == 1)
        {
            // Odd number: copy the last element of the list to the last of list2
            list2[list2.length -1] = list[list.length - 1];
        }
        // Sort the smaller pieces
        list1 = mergeSort(list1);
        list2 = mergeSort(list2);

        // Merge the results
        return merge(list1, list2);
    }

    /////////////////////////////////////////////////////////
    /**
     * Helper function
     * 
     * Sort items in list from i to j.   We will sort this list in place
     * (i.e., create no new lists).
     * 
     * @param list List to be sorted
     * @param i Lowest index to be sorted (inclusive)
     * @param j Highest index to be sorted (inclusive)
     * @param depth Depth of the recursion (used for printlns only)
     */
    public static void quickSort(int[] list, int i, int j, int depth)
    {
        System.out.println("########################");
        System.out.print("Depth: " + depth + "; ");
        System.out.println("Sorting from " + i + " to " + j);

        if(i >= j)
        {
            // Base case
            System.out.println("Done.");
            return;
        }

        // Remember the boundaries for what we are sorting
        int iOrig = i;
        int jOrig = j;

        // Value around which we will organize the lower and upper regions
        int pivot = list[iOrig];

        System.out.println("Pivot: " + pivot);

        // While the two indices have not crossed, then keep going
        while (i <= j)
        {
            // Scan upwards until we find a point that is at or above the pivot
            while(list[i] < pivot)
            {
                System.out.println(list[i]);
                ++i;
            }

            // Scan downwards until we find a point that is at or below the pivot
            while(list[j] > pivot)
            {
                --j;
            }

            if(i <= j)
            {
                // Now have a value in the lower region that belongs in the upper region
                //  and vice versa: swap them
                int tmp = list[i];
                list[i] = list[j];
                list[j] = tmp;
                ++i;
                --j;
            }
        }

        System.out.println("i, j = " + i + " " + j);
        // Now the lower and upper regions fall on either side of the pivot: sort the
        //  two sub-regions
        display(list);

        // Recursive calls
        quickSort(list, iOrig, j, depth + 1);
        quickSort(list, i, jOrig, depth + 1);
    }

    /**
     * Main interface to quickSort
     * 
     * @param list List to be sorted
     */
    public static void quickSort(int[] list)
    {
        quickSort(list, 0, list.length-1 , 1);
    }

    /////////////////////////////////////////////////////////
    public static void display(int[] list)
    {
        for(int i: list)
            System.out.print(i + " ");
        System.out.println("");
    }

    /////////////////////////////////////////////////////////
    public static void main(String[] args)
    {
        int[] list = {123, 2, 8917, 16, 1108, 1723, 113, 1, 17823, 23, 2};
        display(list);

        //list = mergeSort(list);
        quickSort(list);
        display(list);

        System.exit(0);

        // Reverse the sorted list
        for(int i = 0; i < list.length / 2; ++i)
        {
            // Swap opposite sides
            int tmp = list[i];
            list[i] = list[list.length - 1 - i];
            list[list.length - 1 - i] = tmp;
        }
        display(list);


        // With our choice of pivot, the worst case is to sort an already sorted list
        //  (or the reverse of an already sorted list)
        quickSort(list);
        display(list);

    }

}
